------------------------------------------------------
-- DisenchantPredictor.lua
-- Version 1700.2
------------------------------------------------------

-- Configuration
FDP_Config = { };
FDP_Config.Tooltip = true;
FDP_Config.Reagents = true;
FDP_Config.Items = true;
FDP_Config.Verbose = true;

-- formula for dust by level: max(1, ceil((level - 10) / 10))
FDP_DUST_LIST = {
	"Strange Dust",		-- 1-20
	"Soul Dust",		-- 21-30
	"Vision Dust",		-- 31-40
	"Dream Dust",		-- 41-50
	"Illusion Dust",	-- 51-60
}

-- formula for essence by level: max(1, ceil((level - 5) / 5))
FDP_ESSENCE_LIST = {
	"Lesser Magic Essence",		-- 1-10
	"Greater Magic Essence",	-- 11-15
	"Lesser Astral Essence",	-- 16-20
	"Greater Astral Essence",	-- 21-25
	"Lesser Mystic Essence",	-- 26-30
	"Greater Mystic Essence",	-- 31-35
	"Lesser Nether Essence",	-- 36-40
	"Greater Nether Essence",	-- 41-45
	"Lesser Eternal Essence",	-- 46-50
	"Greater Eternal Essence",	-- 51-55 (51-60 is the same)
	"Greater Eternal Essence",	-- 56-60 (51-60 is the same)
}
	
-- formula for shard by level: max(1, ceil((level - 15) / 5))
FDP_SHARD_LIST = {
	"Small Glimmering Shard",	-- 1-20
	"Large Glimmering Shard",	-- 21-25
	"Small Glowing Shard",		-- 26-30
	"Large Glowing Shard",		-- 31-35
	"Small Radiant Shard",		-- 36-40
	"Large Radiant Shard",		-- 41-45
	"Small Brilliant Shard",	-- 46-50
	"Large Brilliant Shard",	-- 51-55 (51-60 is the same)
	"Large Brilliant Shard",	-- 56-60 (51-60 is the same)
}

FDP_DUST_LEVELS = {
	"1-20", "21-30", "31-40", "41-50", "51-60",
}

FDP_ESSENCE_LEVELS = {
	"1-10", "11-15", "16-20", "21-25", "26-30", "31-35", "36-40", "41-45", "46-50", "51-60", "51-60",
}
	
FDP_SHARD_LEVELS = {
	"1-20", "21-25", "26-30", "31-35", "36-40", "41-45", "46-50", "51-60", "51-60",
}

FDP_ArmorSlots = { BACKSLOT, CHESTSLOT, FEETSLOT, FINGER0SLOT, FINGER1SLOT, HANDSSLOT, HEADSLOT, LEGSSLOT, NECKSLOT, SECONDARYHANDSLOT, SHIRTSLOT, SHOULDERSLOT, TABARDSLOT, TRINKET0SLOT, TRINKET1SLOT, WAISTSLOT, WRISTSLOT };

FDP_COLOR_ARTIFACT = "ffffcc9d"; -- untested, extrapolated from ITEM_QUALITY_COLORS
FDP_COLOR_LEGENDARY = "ffff9d00"; -- untested, extrapolated from ITEM_QUALITY_COLORS
FDP_COLOR_EPIC = "ffa335ee";
FDP_COLOR_RARE = "ff0070dd";
FDP_COLOR_UNCOMMON = "ff1eff00";
FDP_COLOR_COMMON = "ffffffff";
FDP_COLOR_POOR = "ff9d9d9d";

local function FDP_Tooltip_Hook(frame, name, link, count, data)
	FDP_Old_Tooltip_Hook(frame, name, link, count, data);

	if ( FDP_Config.Tooltip ) then
		
		if (FDP_Config.Reagents) then
			local kind, index, levelRange;
			if (string.find(name, "^%w+ Dust$")) then
				kind = "DUST";
				index = GFWTable.IndexOf(FDP_DUST_LIST, name);
				levelRange = FDP_DUST_LEVELS[index];
			elseif (string.find(name, "^Lesser %w+ Essence$") or string.find(name, "^Greater %w+ Essence$")) then
				kind = "ESSENCE";
				index = GFWTable.IndexOf(FDP_ESSENCE_LIST, name);
				levelRange = FDP_ESSENCE_LEVELS[index];
			elseif (string.find(name, "^Small %w+ Shard$") or string.find(name, "^Large %w+ Shard$")) then
				kind = "SHARD";
				index = GFWTable.IndexOf(FDP_SHARD_LIST, name);
				levelRange = FDP_SHARD_LEVELS[index];
			end
			if (index ~= nil and index ~= 0) then
				frame.eDone = 1;
				frame:AddLine("Can disenchant from level "..levelRange.." items.", 0.7,0.7,0.7);
				if (FDP_Config.Verbose) then
					if (kind == "SHARD") then
						frame:AddLine("Always disenchants from blue or better items, rarely from green items.", 0.7,0.7,0.7);
					elseif (kind == "ESSENCE") then
						frame:AddLine("Often disenchants from green weapons, occasionally from other green items.", 0.7,0.7,0.7);
					elseif (kind == "DUST") then
						frame:AddLine("Frequently disenchants from any green items.", 0.7,0.7,0.7);
					end
				end
				
				frame:Show();
			end
			return;
		end
		
		if (FDP_Config.Items) then
			local color, level, kind = FDP_ItemInfoForLink(link);
			local dustIndex, essenceIndex, shardIndex;
			if (level ~= nil and kind ~= nil) then
				dustIndex, essenceIndex, shardIndex = FDP_DisenchantByLevel(level);
				local dustName = FDP_DUST_LIST[dustIndex];
				local essenceName = FDP_ESSENCE_LIST[essenceIndex];
				local shardName  = FDP_SHARD_LIST[shardIndex];
				if (color == FDP_COLOR_RARE or color == FDP_COLOR_EPIC or color == FDP_COLOR_LEGENDARY or color == FDP_COLOR_ARTIFACT) then
					frame.eDone = 1;
					frame:AddLine("Always disenchants to "..shardName..".", GFW_FONT_COLOR.r, GFW_FONT_COLOR.g, GFW_FONT_COLOR.b);
					frame:Show();
				elseif (color == FDP_COLOR_UNCOMMON) then
					frame.eDone = 1;
					frame:AddLine("Can disenchant to:", GFW_FONT_COLOR.r, GFW_FONT_COLOR.g, GFW_FONT_COLOR.b);
					frame:AddLine("  "..dustName.." (most likely)", GFW_FONT_COLOR.r, GFW_FONT_COLOR.g, GFW_FONT_COLOR.b);
					if (kind == "WEAPON") then
						frame:AddLine("  "..essenceName.." (likely)", GFW_FONT_COLOR.r, GFW_FONT_COLOR.g, GFW_FONT_COLOR.b);
					else
						frame:AddLine("  "..essenceName.." (occasionally)", GFW_FONT_COLOR.r, GFW_FONT_COLOR.g, GFW_FONT_COLOR.b);		
					end
					frame:AddLine("  "..shardName.." (rarely)", GFW_FONT_COLOR.r, GFW_FONT_COLOR.g, GFW_FONT_COLOR.b);
					frame:Show();
				end
			end
		end
	end
	
end

function FDP_OnLoad()

	-- Register Slash Commands
	SLASH_FDP1 = "/enchant";
	SLASH_FDP2 = "/disenchant";
	SLASH_FDP3 = "/ench";
	SLASH_FDP4 = "/dis";
	SLASH_FDP5 = "/de";
	SLASH_FDP6 = "/dp";
	SlashCmdList["FDP"] = function(msg)
		FDP_ChatCommandHandler(msg);
	end
	
	FDP_Old_Tooltip_Hook = TT_AddTooltip;
	TT_AddTooltip = FDP_Tooltip_Hook;

	GFWUtils.Print("Fizzwidget Disenchant Predictor initialized!");
end

function FDP_ChatCommandHandler(msg)

	-- Print Help
	if ( msg == "help" ) or ( msg == "" ) then
		GFWUtils.Print("Fizzwidget Disenchant Predictor:");
		GFWUtils.Print("/enchant (or /ench or /disenchant or /dis or /de)");
		GFWUtils.Print("- "..GFWUtils.Hilite("help").." - Print this helplist.");
		GFWUtils.Print("- "..GFWUtils.Hilite("status").." - Check current settings.");
		GFWUtils.Print("- "..GFWUtils.Hilite("reagents on").." | "..GFWUtils.Hilite("off").." - add info to tooltips for enchanting reagents (e.g. Strange Dust) showing what they can disenchant from.");
		GFWUtils.Print("- "..GFWUtils.Hilite("items on").." | "..GFWUtils.Hilite("off").." - add info to tooltips for disenchantable items (e.g. green weapons) showing what they can disenchant into.");
		GFWUtils.Print("- "..GFWUtils.Hilite("tooltip on").." | "..GFWUtils.Hilite("off").." - turns all tooltip modificaitons on and off.");
		GFWUtils.Print("- "..GFWUtils.Hilite("verbose on").." | "..GFWUtils.Hilite("off").." - add an extra line to tooltips for enchanting reagents further explaining how to get them from disenchanting.");
		GFWUtils.Print("- "..GFWUtils.Hilite("<level number>").." - provide an item's level requirement to see what it can disenchant to.");
		GFWUtils.Print("- "..GFWUtils.Hilite("<item link>").." - shift-click an item to see what it can disenchant to.");
		return;
	end
	
	if (msg == "reagents on") then
		FDP_Config.Reagents = true;
		FDP_Config.Tooltip = true;
		GFWUtils.Print("Showing disenchant level range on tooltips for enchanting reagents.");
		return;
	end
	if (msg == "reagents off") then
		FDP_Config.Reagents = false;
		GFWUtils.Print("Not adding info to tooltips for enchanting reagents.");
		return;
	end
	if (msg == "items on") then
		FDP_Config.Items = true;
		FDP_Config.Tooltip = true;
		GFWUtils.Print("Showing disenchant predictions on tooltips for disenchantable items.");
		return;
	end
	if (msg == "items off") then
		FDP_Config.Items = false;
		GFWUtils.Print("Not adding any info to tooltips for disenchantable items.");
		return;
	end
	if (msg == "tooltip on") then
		FDP_Config.Tooltip = true;
		GFWUtils.Print("Adding info to tooltips.");
		return;
	end
	if (msg == "tooltip off") then
		FDP_Config.Tooltip = false;
		GFWUtils.Print("Not adding any info to tooltips.");
		return;
	end
	if (msg == "verbose on") then
		FDP_Config.Verbose = true;
		GFWUtils.Print("Also showing more specific info for certain enchanting reagent tooltips.");
		return;
	end
	if (msg == "verbose off") then
		FDP_Config.Verbose = false;
		GFWUtils.Print("Not showing more specific info for enchanting reagents.");
		return;
	end
	
	if ( msg == "status" ) then
		if ( FDP_Config.Tooltip and (FDP_Config.Items or FDP_Config.Reagents)) then
			if (FDP_Config.Items) then
				GFWUtils.Print("Showing disenchant predictions on tooltips for disenchantable items.");
			end
			if (FDP_Config.Reagents) then
				GFWUtils.Print("Showing disenchant level range on tooltips for enchanting reagents.");
				if ( FDP_Config.Verbose ) then
					GFWUtils.Print("Also showing more specific info for certain enchanting reagent tooltips.");
				else
					GFWUtils.Print("Not showing more specific info for enchanting reagents.");
				end
			end
		else
			GFWUtils.Print("Not adding any info to tooltips.");
		end
		return;
	end

	if ( tonumber(msg) ~= nil ) then
		local level = tonumber(msg);
		if (level < 1 or level > 60) then
			GFWUtils.Print("Item levels must be between 1 and 60.");
		end
		local dustIndex, essenceIndex, shardIndex = FDP_DisenchantByLevel(level);
		GFWUtils.Print("An item of level "..GFWUtils.Hilite(level).." can disenchant into:");
		GFWUtils.Print(GFWUtils.Hilite(FDP_DUST_LIST[dustIndex]).." (likely for green items)");
		GFWUtils.Print(GFWUtils.Hilite(FDP_ESSENCE_LIST[essenceIndex]).." (likely for green weapons, occasionally for other green items)");
		GFWUtils.Print(GFWUtils.Hilite(FDP_SHARD_LIST[shardIndex]).." (rarely for green items, always for blue or better items)");
		return;
	end
	
	local _, _, link  = string.find(msg, "(.c%x+.Hitem:%d+:%d+:%d+:%d+.h%[[^]]+%].h.r)");
	if (link ~= nil and link ~= "") then
		local color, level, kind = FDP_ItemInfoForLink(link);
		
		local colorName;
		if (color == FDP_COLOR_COMMON or color == FDP_COLOR_POOR) then
			GFWUtils.Print(link .." most likely can't be disenchanted (only items of green or better quality can).");
		elseif (color == FDP_COLOR_UNCOMMON) then
			colorName = "GREEN";
		elseif (color == FDP_COLOR_RARE) then
			colorName = "BLUE";
		elseif (color == FDP_COLOR_EPIC or color == FDP_COLOR_LEGENDARY or color == FDP_COLOR_ARTIFACT) then
			colorName = "PURPLE+";
		end
		
		local dustIndex, essenceIndex, shardIndex;
		if (level ~= nil) then
			dustIndex, essenceIndex, shardIndex = FDP_DisenchantByLevel(level);
			local dustName = FDP_DUST_LIST[dustIndex];
			local essenceName = FDP_ESSENCE_LIST[essenceIndex];
			local shardName  = FDP_SHARD_LIST[shardIndex];
			if (colorName == "BLUE" and kind ~= nil) then
				GFWUtils.Print(link .." will disenchant to "..GFWUtils.Hilite(shardName)..".");
			elseif (colorName == "PURPLE+" and kind ~= nil) then
				GFWUtils.Print(link .." will disenchant to "..GFWUtils.Hilite(shardName).." (likely mulitple).");
			elseif (colorName == "GREEN" and kind ~= nil) then
				GFWUtils.Print(link .." can disenchant to any of:");
				GFWUtils.Print(" - "..GFWUtils.Hilite(dustName).." (most likely)");
				if (kind == "WEAPON") then
					GFWUtils.Print(" - "..GFWUtils.Hilite(essenceName).." (likely)");
				else
					GFWUtils.Print(" - "..GFWUtils.Hilite(essenceName).." (occasionally)");		
				end
				GFWUtils.Print(" - "..GFWUtils.Hilite(shardName).." (rarely)");
				return;
			end
		else
			if (kind == nil) then
				GFWUtils.Print(link .." may not be disenchantable. (Generally, only equippable, non-stackable items are.)");
			else
				if (colorName == "BLUE" or colorName == "PURPLE+") then
					GFWUtils.Print(link .." has no level requirement; it will produce a "..GFWUtils.Hilite("shard").." when disenchanted, but I can't predict which kind. (This is likely a quest reward item, and thus likely to disenchant as would an item whose required level is similar to that of the quest.)");
				else -- green
					GFWUtils.Print(link .." has no level requirement; I can't predict which kind of dust/essence/shard will be produced when it is disenchanted. (This is likely a quest reward item, and thus likely to disenchant as would an item whose required level is similar to that of the quest.)");
					if (kind == "WEAPON") then
						GFWUtils.Print("Being a weapon, this item is more likely to disenchant to an essence than other items are. Dust is most likely, and shards are rare.");
					else
						GFWUtils.Print("Dust is most likely, an essence is uncommon, and shards are rare.");
					end
				end
				return;
			end
		end
		GFWUtils.Print("Can't determine disenchant info from "..link ..".");
	end
end

function FDP_DisenchantByLevel(level)
    local dustIndex = math.max(1, math.ceil((level - 10) / 10));
    local essenceIndex = math.max(1, math.ceil((level - 5) / 5));
    local shardIndex = math.max(1, math.ceil((level - 15) / 5));
    return dustIndex, essenceIndex, shardIndex;
end

function FDP_ItemInfoForLink(itemLink)
--ITEM_MIN_LEVEL = "Requires Level %d"; -- Required level to use the item
--ITEM_MIN_SKILL = "Requires %s (%d)"; -- Required skill rank to use the item
	
	local _, _, color, link = string.find(itemLink, ".c(%x+).H(item:%d+:%d+:%d+:%d+).h%[.-%].h.r");
	if (link == nil or link ~= "") then return nil; end
	FDPHiddenTooltip:SetHyperlink(link);
	
	local level, skill, kind;
	for lineNum = 1, FDPHiddenTooltip:NumLines() do
		local leftText = getglobal("FDPHiddenTooltipTextLeft"..lineNum):GetText();
		local _, _, levelString = string.find(leftText, "Requires Level (%d+)");
		if (levelString ~= nil and tonumber(levelString) ~= nil) then
			level = tonumber(levelString);
		end
		local _, _, skillString = string.find(leftText, "Requires %w+[ ]*%w* %((%d+)%)");
		if (levelString == nil and skillString ~= nil and tonumber(skillString) ~= nil) then
			skill = tonumber(skillString);
			if (lineNum < 4) then
				kind = "RECIPE"; 
				-- if it's got a skill level on one of the first couple of lines and no armor slot or speed, it's very likely a recipe
			end
		end
		for _, slotName in FDP_ArmorSlots do
			if (leftText == slotName) then
				kind = "ARMOR";
			end
			local rightText = getglobal("FDPHiddenTooltipTextRight"..lineNum):GetText();
			if (rightText ~= nil and string.find(rightText, "Speed %d+\.%d+")) then
				kind = "WEAPON"; -- all weapons have a speed
			end
		end
	end
	if ((skill and level) or kind == "RECIPE") then
		return nil; -- if there's both a skill requirement and a level requirement, it's probably a recipe (which can't be DE'ed)
	end
	return color, level, kind;
end
